/*
 * MaFenetre.cpp
 * This file is part of ChronoCheckMate
 *
 * Copyright (C) 2010 - François Drouhard
 *
 * ChronoCheckMate is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * ChronoCheckMate is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with ChronoCheckMate; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, 
 * Boston, MA  02110-1301  USA
 */
 
#include "MaFenetre.h"
#include "Chronometre.h"
#include "options.h"
#include "Dialog.h"
#include <QMenuBar>
#include <QHBoxLayout>
#include <QVBoxLayout>
#include <QPushButton>
#include <QKeyEvent>
#include <QApplication>
#include <QLabel>
#include <QDialog>
#include <QLineEdit>
#include <QDialogButtonBox>
#include <QMessageBox>
#include <QSound>

MaFenetre::MaFenetre(Options* mesOptions) :
    QMainWindow (),
    marche(false),
    option(mesOptions)
{
    QWidget *centralWidget = new QWidget;

    chronoBlanc = new Chronometre(option->nomJoueur1() , option->tempsDepart(), this);
    chronoNoir = new Chronometre(option->nomJoueur2() , option->tempsDepart(), this);
    boutonInverser = new QPushButton ("<->");
    boutonInverser->setMinimumWidth(50);
    boutonInverser->setMaximumWidth(50);

	////////////////// Groupe Définition de la classe ///////////////////////////
    layoutChrono = new QHBoxLayout;
    layoutChrono->addWidget(chronoBlanc);
    layoutChrono->addWidget(boutonInverser, Qt::AlignVCenter);
    layoutChrono->addWidget(chronoNoir);

    ///////////////////////////// Menus ////////////////////////////
    QMenu *menuFichier = menuBar()->addMenu("&Fichier");
    QAction *actionNouveau = new QAction ("&Nouveau" , this);
    QAction *actionOptions = new QAction ("&Options...", this);
    QAction *actionQuitter = new QAction ("&Quitter" , this);
    menuFichier->addAction(actionNouveau);
    menuFichier->addAction(actionOptions);
    menuFichier->addSeparator();
    menuFichier->addAction(actionQuitter);

	///////////////////////////// Boutons   /////////////////////////////
    boutonDemarrer = new QPushButton ("Commencer (Barre Espace)");
    boutonArreter = new QPushButton ("Arrêter(Touche Entrée)");
	
	boutonArreter->setFocusPolicy (Qt::NoFocus);
    boutonInverser->setFocusPolicy(Qt::NoFocus);
	
	layoutBoutonSwitch = new QHBoxLayout;
	layoutBoutonSwitch->addWidget (boutonDemarrer);
    layoutBoutonSwitch->addWidget (boutonArreter);

    ///////////////////////////// Aide /////////////////////////////////////
    QLabel *labelAide = new QLabel ("<b>Aide</b><br />Espace : Lancer le chrono/Donner le chrono<br />Entrée : Stopper les deux chronos");
	
	/////////////////////////////Mise en page finale ///////////////////////////
	layoutComplet = new QVBoxLayout;
	layoutComplet->addLayout (layoutChrono);
	layoutComplet->addLayout (layoutBoutonSwitch);
    layoutComplet->addWidget (labelAide);
    centralWidget->setLayout (layoutComplet);
    setCentralWidget(centralWidget);
	setWindowTitle ("ChronoCheckMate");

    ///////////////////////////// Initialisation du jeu /////////////////
    //lancerDialogueOptions();
    init();

	///////////////////////////// connections ///////////////////////////
    QObject::connect (boutonDemarrer , SIGNAL(clicked () ) , this , SLOT(demarrer () ));
    QObject::connect (boutonInverser , SIGNAL(clicked()) , this , SLOT(inverser()));
    QObject::connect (boutonArreter , SIGNAL(clicked()), this, SLOT (arreter()));
    QObject::connect (actionNouveau, SIGNAL(triggered()) , this , SLOT(init()));
    QObject::connect (actionOptions , SIGNAL(triggered()), this, SLOT(lancerDialogueOptions()));
    QObject::connect (actionQuitter , SIGNAL(triggered()) , qApp , SLOT(quit()));
    QObject::connect (chronoBlanc, SIGNAL(fin()), this, SLOT(afficheGagnant()));
    QObject::connect (chronoNoir, SIGNAL(fin()), this, SLOT(afficheGagnant()));

}

///////////////// Méthodes ////////////////////
void MaFenetre::etatBoutons (bool etatBoutonStart, bool etatBoutonStop, bool etatBoutonInverser) {
    boutonDemarrer->setEnabled (etatBoutonStart);
    boutonArreter->setEnabled (etatBoutonStop);
    boutonInverser->setEnabled(etatBoutonInverser);
    //boutonInverser->setVisible(etatBoutonInverser);
}

////////////////// SLOTS ////////////////////////
void MaFenetre::inverser() {
    layoutChrono->removeWidget(chronoBlanc);
    layoutChrono->removeWidget(chronoNoir);
    Chronometre *chronoTemp = chronoBlanc;
    chronoBlanc = chronoNoir;
    chronoNoir = chronoTemp;
    layoutChrono->insertWidget(0 , chronoBlanc);
    layoutChrono->addWidget(chronoNoir);
}

void MaFenetre::demarrer () {
	if (marche == false ) {
        chronoBlanc->reinit ();
        chronoNoir->reinit ();
        chronoBlanc->start ();
		marche = true;
        etatBoutons (false , true, false);
	}
}

void MaFenetre::switcher () {
	if (marche == true) {
        chronoBlanc->basculer ();
        chronoNoir->basculer ();
	}
}

void MaFenetre::arreter () {
    chronoBlanc->stop();
    chronoNoir->stop();
	marche = false;
    etatBoutons (false , false, true);
}

void MaFenetre::afficheGagnant() {
    arreter();
    if (option->son() == true)
        QSound::play(":/ressources/sonnette.wav");
    QMessageBox message;
    QString texte;
    texte = "Le joueur " + (*chronoBlanc+=*chronoNoir) + " a gagné au temps.";
    message.setText(texte);
    message.setStandardButtons(QMessageBox::Ok);
    message.exec();
}

void MaFenetre::init () {
    chronoBlanc->reinit();
    chronoNoir->reinit();
    marche = false;
    etatBoutons (true , false, true);
}

void MaFenetre::lancerDialogueOptions() {
    Dialog dialog (option);
    if (dialog.exec()) {
        chronoBlanc->modifierNomJoueur(option->nomJoueur1());
        chronoBlanc->definirTemps(option->tempsDepart());

        chronoNoir->modifierNomJoueur(option->nomJoueur2());
        chronoNoir->definirTemps(option->tempsDepart());

        if (marche==false) {
            init();
        }
    }
}

void MaFenetre::keyReleaseEvent (QKeyEvent * evenement) {
	if (evenement->key() == Qt::Key_Space) {
        if (marche == true) {
            switcher();
        }
        if (marche == false) {
            demarrer();
        }
	}
    if (evenement->key() == Qt::Key_Return) {
        arreter();
    }
}
